<?php

declare(strict_types=1);

namespace Drupal\web_initial_paragraphs\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\web_initial_paragraphs\Plugin\Field\FieldType\WipColorItem;
use Drupal\Core\StringTranslation\TranslatableMarkup;

/**
 * Plugin implementation of the 'WIP Color value' formatter.
 *
 * @extends \Drupal\Core\Field\FormatterBase<\Drupal\Core\Field\FieldItemListInterface<\Drupal\web_initial_paragraphs\Plugin\Field\FieldType\WipColorItem>>
 *
 * @FieldFormatter(
 * id = "wip_color_formatter",
 * label = @Translation("WIP Color Display"),
 * field_types = {
 * "wip_color"
 * }
 * )
 */
final class WipColorFormatter extends FormatterBase {

  /**
   * {@inheritdoc}
   *
   * @return array<string, mixed>
   *   An array of default settings for the formatter.
   */
  public static function defaultSettings(): array {
    return [
      'output_type' => 'visual',
      'display_text' => TRUE,
      'display_swatch' => TRUE,
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   *
   * @param array<string, mixed> $form
   *   An associative array containing the initial form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return array<string, mixed>
   *   The form elements for the formatter settings.
   */
  public function settingsForm(array $form, FormStateInterface $form_state): array {
    $element['output_type'] = [
      '#type' => 'select',
      '#title' => $this->t('Output format'),
      '#options' => [
        'visual' => $this->t('Visual display (swatch and text)'),
        'css_only' => $this->t('Raw CSS value only'),
      ],
      '#default_value' => $this->getSetting('output_type'),
      '#description' => $this->t('Choose "Raw CSS value" to output only the CSS string, useful for inline styles in Twig.'),
    ];

    $element['visual_options'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Visual display options'),
      '#states' => [
        'visible' => [
          ':input[name$="[output_type]"]' => ['value' => 'visual'],
        ],
      ],
    ];

    $element['visual_options']['display_swatch'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Display color swatch'),
      '#default_value' => $this->getSetting('display_swatch'),
    ];

    $element['visual_options']['display_text'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Display color value text'),
      '#default_value' => $this->getSetting('display_text'),
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   *
   * @return array<int, string|TranslatableMarkup>
   *   An array of summary strings for the formatter settings.
   */
  public function settingsSummary(): array {
    $summary = [];
    $output_type = $this->getSetting('output_type');
    $summary[] = $this->t('Format: @type', ['@type' => $output_type === 'visual' ? $this->t('Visual') : $this->t('Raw CSS')]);

    if ($output_type === 'visual') {
      if ($this->getSetting('display_swatch')) {
        $summary[] = $this->t('Swatch: Yes');
      }
      if ($this->getSetting('display_text')) {
        $summary[] = $this->t('Text: Yes');
      }
    }

    return $summary;
  }

  /**
   * {@inheritdoc}
   *
   * @param \Drupal\Core\Field\FieldItemListInterface<\Drupal\web_initial_paragraphs\Plugin\Field\FieldType\WipColorItem> $items
   *   The field items to be rendered.
   * @param string $langcode
   *   The language code.
   *
   * @return array<int, array<string, mixed>>
   *   A render array for the field.
   */
  public function viewElements(FieldItemListInterface $items, $langcode): array {
    $elements = [];
    $output_type = $this->getSetting('output_type');

    foreach ($items as $delta => $item) {
      $css_value = $this->buildCssValue($item);

      if (empty($css_value)) {
        continue;
      }

      if ($output_type === 'css_only') {
        $elements[$delta] = [
          '#plain_text' => $css_value,
        ];
      }
      else {
        $elements[$delta] = [
          '#type' => 'container',
          '#attributes' => ['class' => ['wip-color-formatter-wrapper']],
          'swatch' => [
            '#type' => 'html_tag',
            '#tag' => 'span',
            '#attributes' => [
              'style' => 'background: ' . $css_value . '; display:inline-block; width:20px; height:20px; border:1px solid #ccc; vertical-align: middle; margin-right: 5px;',
              'aria-hidden' => 'true',
              'title' => $css_value,
            ],
            '#access' => (bool) $this->getSetting('display_swatch'),
          ],
          'text' => [
            '#plain_text' => $css_value,
            '#prefix' => '<span class="wip-color-text">',
            '#suffix' => '</span>',
            '#access' => (bool) $this->getSetting('display_text'),
          ],
        ];
      }
    }

    return $elements;
  }

  /**
   * Builds a CSS color value string based on the provided item properties.
   *
   * @param \Drupal\web_initial_paragraphs\Plugin\Field\FieldType\WipColorItem $item
   *   The color item object containing color type and value properties:
   *   - wip_color_type: (string) The type of color ('solid' or 'gradient').
   *   - wip_value: (string) The main color value (hex, rgb, etc.).
   *   - wip_end_color: (string, optional) The end color for gradients.
   *   - wip_direction: (string, optional) The direction for gradients.
   *
   * @return string
   *   The CSS color value string, or an empty string if the input is invalid.
   */
  private function buildCssValue(object $item): string {
    $type = $item->wip_color_type ?? 'solid';

    if ($type === 'solid' && !empty($item->wip_value)) {
      return $this->formatColorForCss($item->wip_value);
    }

    if ($type === 'gradient') {
      $start = $item->wip_value;
      $end = $item->wip_end_color;
      $direction = $item->wip_direction ?? 'to right';

      if ($start && $end) {
        $start_css = $this->formatColorForCss($start);
        $end_css = $this->formatColorForCss($end);
        return "linear-gradient({$direction}, {$start_css}, {$end_css})";
      }
    }

    return '';
  }

  /**
   * Formats a hexadecimal color string for use in CSS.
   *
   * This method accepts a color string in the format #RRGGBBAA or #RRGGBB.
   * - If the color is empty, returns an empty string.
   * - If the color includes an alpha channel (AA) and is fully opaque (FF),
   *   returns the color in #RRGGBB format.
   * - If the color includes an alpha channel with transparency, converts it
   *   to an rgba() CSS string for compatibility.
   * - Otherwise, returns the original color string.
   *
   * @param string|null $color
   *   The hexadecimal color string, possibly including alpha (e.g., #RRGGBBAA).
   *
   * @return string
   *   The formatted color string for CSS usage.
   */
  private function formatColorForCss(?string $color): string {
    if (empty($color)) {
      return '';
    }

    // If the opacity is full opacity (FF), the format #RRGGBB can be used.
    if (strlen($color) === 9 && strtoupper(substr($color, -2)) === 'FF') {
      return substr($color, 0, 7);
    }

    // If it has an alpha channel
    // it is converted to rgba() for maximum compatibility.
    if (strlen($color) === 9) {
      $r = hexdec(substr($color, 1, 2));
      $g = hexdec(substr($color, 3, 2));
      $b = hexdec(substr($color, 5, 2));
      $a = round(hexdec(substr($color, 7, 2)) / 255, 2);
      return "rgba({$r}, {$g}, {$b}, {$a})";
    }

    return $color;
  }

}
