<?php

declare(strict_types=1);

namespace Drupal\web_initial_paragraphs\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\web_initial_paragraphs\Plugin\Field\FieldType\WipColorItem;

/**
 * Defines the 'wip_color_widget' field widget.
 *
 * @FieldWidget(
 *   id = "wip_color_widget",
 *   label = @Translation("WIP Color picker"),
 *   field_types = {
 *     "wip_color"
 *   },
 *   )
 */
final class WipColorWidget extends WidgetBase {

  /**
   * {@inheritdoc}
   *
   * Builds the form element for a single field item.
   * This method renders an HTML5 <input type="color"> element.
   *
   * @param \Drupal\Core\Field\FieldItemListInterface<\Drupal\web_initial_paragraphs\Plugin\Field\FieldType\WipColorItem> $items
   *   The field items being rendered.
   * @param int $delta
   *   The order of this item in the array of field items.
   * @param array<string, mixed> $element
   *   A form element array to populate with the widget's properties.
   * @param array<string, mixed> $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return array<string, mixed>
   *   The form element array for the widget.
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state): array {

    $element['#type'] = 'container';

    $element['#attributes'] = is_array($element['#attributes'] ?? NULL) ? $element['#attributes'] : [];
    $element['#attributes']['class'] = is_array($element['#attributes']['class'] ?? NULL)
      ? $element['#attributes']['class']
      : [];
    $element['#attributes']['class'][] = 'wip-color-widget-container';

    $element['#attached'] = is_array($element['#attached'] ?? NULL) ? $element['#attached'] : [];
    $element['#attached']['library'] = is_array($element['#attached']['library'] ?? NULL)
      ? $element['#attached']['library']
      : [];
    $element['#attached']['library'][] = 'web_initial_paragraphs/wip_color_widget_ux';

    $element['wip_color_type'] = [
      '#type' => 'radios',
      '#title' => new TranslatableMarkup('Color Type'),
      '#options' => [
        'solid' => new TranslatableMarkup('Solid'),
        'gradient' => new TranslatableMarkup('Gradient'),
      ],
      '#default_value' => $items[$delta]->wip_color_type ?? 'solid',
      '#attributes' => [
        'class' => ['wip-color-type-selector'],
      ],
    ];

    $parents = array_merge(
      (array) $element['#field_parents'],
      [
        $this->fieldDefinition->getName(),
        $delta,
        'wip_color_type',
      ]
    );

    $selector_value = array_shift($parents);
    $selector_name = is_string($selector_value) ? $selector_value : '';
    $selector_name .= $parents ? '[' . implode('][', $parents) . ']' : '';
    $selector_path = ':input[name="' . $selector_name . '"]';

    $element['solid_container'] = [
      '#type' => 'container',
      '#attributes' => ['class' => ['wip-color-solid-container']],
      '#states' => ['visible' => [$selector_path => ['value' => 'solid']]],
    ];

    $element['solid_container']['wip_value'] = [
      '#type' => 'color',
      '#title' => new TranslatableMarkup('Color'),
      '#default_value' => $this->formatHexForInput($items[$delta]->wip_value ?? ''),
    ];

    $element['gradient_container'] = [
      '#type' => 'container',
      '#title' => new TranslatableMarkup('Gradient Configuration'),
      '#attributes' => ['class' => ['wip-color-gradient-container']],
      '#states' => ['visible' => [$selector_path => ['value' => 'gradient']]],
    ];

    $element['gradient_container']['wip_value'] = [
      '#type' => 'color',
      '#title' => new TranslatableMarkup('Start Color'),
      '#default_value' => $this->formatHexForInput($items[$delta]->wip_value ?? ''),
    ];

    $element['gradient_container']['wip_end_color'] = [
      '#type' => 'color',
      '#title' => new TranslatableMarkup('End Color'),
      '#default_value' => $this->formatHexForInput($items[$delta]->wip_end_color ?? ''),
    ];

    $element['gradient_container']['wip_direction'] = [
      '#type' => 'select',
      '#title' => new TranslatableMarkup('Direction'),
      '#description' => new TranslatableMarkup('Enter a value like "180deg", "to right", or "to top left".'),
      '#options' => [
        'to right' => $this->t('To right'),
        'to left' => $this->t('To left'),
        'to bottom' => $this->t('To bottom'),
        'to top' => $this->t('To top'),
        'to bottom right' => $this->t('To bottom right'),
        'to bottom left' => $this->t('To bottom left'),
        'to top right' => $this->t('To top right'),
        'to top left' => $this->t('To top left'),
        '45deg' => $this->t('45 degrees'),
        '90deg' => $this->t('90 degrees'),
        '135deg' => $this->t('135 degrees'),
        '180deg' => $this->t('180 degrees'),
      ],
      '#default_value' => (string) ($items[$delta]->wip_direction ?? 'to right'),
      '#states' => [
        'required' => [$selector_path => ['value' => 'gradient']],
      ],
    ];

    unset($element['value']);

    return $element;
  }

  /**
   * {@inheritdoc}
   *
   * @param array<int, array<string, mixed>> $values
   *   The array of form values to massage.
   * @param array<string, mixed> $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return array<int, array<string, mixed>>
   *   The massaged array of form values.
   */
  public function massageFormValues(array $values, array $form, FormStateInterface $form_state): array {
    $new_values = [];
    foreach ($values as $value) {
      /** @var array<string, mixed> $value */
      $wip_color_type = $value['wip_color_type'] ?? 'solid';
      $processed_value = ['wip_color_type' => $wip_color_type];

      if ($wip_color_type === 'solid') {
        $solid_container = (array) ($value['solid_container'] ?? []);
        $wip_value_raw = $solid_container['wip_value'] ?? NULL;
        $wip_value = is_string($wip_value_raw) ? $wip_value_raw : NULL;
        $processed_value['wip_value'] = $this->normalizeHex($wip_value);
      }
      elseif ($wip_color_type === 'gradient') {
        $gradient_container = (array) ($value['gradient_container'] ?? []);
        $wip_value_raw = $gradient_container['wip_value'] ?? NULL;
        $wip_end_color_raw = $gradient_container['wip_end_color'] ?? NULL;

        $wip_value = is_string($wip_value_raw) ? $wip_value_raw : NULL;
        $wip_end_color = is_string($wip_end_color_raw) ? $wip_end_color_raw : NULL;

        $processed_value['wip_value'] = $this->normalizeHex($wip_value);
        $processed_value['wip_end_color'] = $this->normalizeHex($wip_end_color);
        $processed_value['wip_direction'] = $gradient_container['wip_direction'] ?? NULL;
      }
      $new_values[] = $processed_value;
    }
    return $new_values;
  }

  /**
   * Normalizes a hexadecimal color string to the format #RRGGBBAA.
   *
   * This method ensures that the input color string:
   * - Starts with a '#' character.
   * - Is converted to uppercase.
   * - If the color is in the #RRGGBB format (7 characters)
   *   appends 'FF' for full opacity.
   * - Returns the color in #RRGGBBAA format (9 characters) or NULL if invalid.
   *
   * @param string|null $color
   *   The input color string, which may be NULL or in various formats.
   *
   * @return string|null
   *   The normalized color string in #RRGGBBAA format,
   *   or NULL if the input is invalid.
   */
  private function normalizeHex(?string $color): ?string {
    if (empty($color)) {
      return NULL;
    }
    if (substr($color, 0, 1) !== '#') {
      $color = '#' . $color;
    }
    $color = strtoupper($color);
    // If it is a 7-character RGB colour, add FF for full opacity.
    if (strlen($color) === 7) {
      $color .= 'FF';
    }
    return (strlen($color) === 9) ? $color : NULL;
  }

  /**
   * Formats a 9-character hex color string (with alpha) for input fields.
   *
   * If the provided hex color string is 9 characters long and starts with '#',
   * it trims the string to the first 7 characters (removing the alpha channel).
   * Otherwise, it returns the original hex color or an empty string if null.
   *
   * @param string|null $hex_color
   *   The hex color string, possibly including
   *   an alpha channel (e.g., '#RRGGBBAA').
   *
   * @return string
   *   The formatted hex color string suitable for input fields
   *   (e.g., '#RRGGBB'), or an empty string if the input is null.
   */
  protected function formatHexForInput(?string $hex_color): string {
    if ($hex_color && strlen($hex_color) === 9 && substr($hex_color, 0, 1) === '#') {
      return substr($hex_color, 0, 7);
    }
    return $hex_color ?? '';
  }

}
